@@ -1,6 +1,10 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package gcc
 
 import (
+	"runtime"
 	"testing"
 	"time"
 
@@ -16,6 +20,7 @@ func (t staticThreshold) compare(estimate, _ time.Duration) (usage, time.Duratio
 	if estimate < -time.Duration(t) {
 		return usageUnder, estimate, time.Duration(t)
 	}
+
 	return usageNormal, estimate, time.Duration(t)
 }
 
@@ -95,7 +100,12 @@ func TestOveruseDetectorWithoutDelay(t *testing.T) {
 				defer close(out)
 				for _, e := range tc.estimates {
 					od.onDelayStats(e)
-					time.Sleep(tc.delay)
+					if tc.delay == 0 {
+						// avoid time.Sleep(0) since it's broken on windows.
+						runtime.Gosched()
+					} else {
+						time.Sleep(tc.delay)
+					}
 				}
 			}()
 			received := []usage{}
