@@ -1,20 +1,24 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package twcc
 
 import (
-	"fmt"
 	"testing"
 
 	"github.com/pion/rtcp"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 func rtcpToTwcc(t *testing.T, in []rtcp.Packet) []*rtcp.TransportLayerCC {
+	t.Helper()
+
 	out := make([]*rtcp.TransportLayerCC, len(in))
 	var ok bool
 	for i, pkt := range in {
-		if out[i], ok = pkt.(*rtcp.TransportLayerCC); !ok {
-			t.Fatal("Failed to cast")
-		}
+		out[i], ok = pkt.(*rtcp.TransportLayerCC)
+		assert.True(t, ok, "Expected TransportLayerCC, got %T", pkt)
 	}
 
 	return out
@@ -22,20 +26,20 @@ func rtcpToTwcc(t *testing.T, in []rtcp.Packet) []*rtcp.TransportLayerCC {
 
 func Test_chunk_add(t *testing.T) {
 	t.Run("fill with not received", func(t *testing.T) {
-		c := &chunk{}
+		testChunk := &chunk{}
 
 		for i := 0; i < maxRunLengthCap; i++ {
-			assert.True(t, c.canAdd(rtcp.TypeTCCPacketNotReceived), i)
-			c.add(rtcp.TypeTCCPacketNotReceived)
+			assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived), i)
+			testChunk.add(rtcp.TypeTCCPacketNotReceived)
 		}
-		assert.Equal(t, make([]uint16, maxRunLengthCap), c.deltas)
-		assert.False(t, c.hasDifferentTypes)
+		assert.Equal(t, make([]uint16, maxRunLengthCap), testChunk.deltas)
+		assert.False(t, testChunk.hasDifferentTypes)
 
-		assert.False(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
-		assert.False(t, c.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
-		assert.False(t, c.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
+		assert.False(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
+		assert.False(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
+		assert.False(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
 
-		statusChunk := c.encode()
+		statusChunk := testChunk.encode()
 		assert.IsType(t, &rtcp.RunLengthChunk{}, statusChunk)
 
 		buf, err := statusChunk.Marshal()
@@ -44,20 +48,20 @@ func Test_chunk_add(t *testing.T) {
 	})
 
 	t.Run("fill with small delta", func(t *testing.T) {
-		c := &chunk{}
+		testChunk := &chunk{}
 
 		for i := 0; i < maxOneBitCap; i++ {
-			assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta), i)
-			c.add(rtcp.TypeTCCPacketReceivedSmallDelta)
+			assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta), i)
+			testChunk.add(rtcp.TypeTCCPacketReceivedSmallDelta)
 		}
 
-		assert.Equal(t, c.deltas, []uint16{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1})
-		assert.False(t, c.hasDifferentTypes)
+		assert.Equal(t, testChunk.deltas, []uint16{1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1})
+		assert.False(t, testChunk.hasDifferentTypes)
 
-		assert.False(t, c.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
-		assert.False(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
+		assert.False(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
+		assert.False(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
 
-		statusChunk := c.encode()
+		statusChunk := testChunk.encode()
 		assert.IsType(t, &rtcp.RunLengthChunk{}, statusChunk)
 
 		buf, err := statusChunk.Marshal()
@@ -66,21 +70,21 @@ func Test_chunk_add(t *testing.T) {
 	})
 
 	t.Run("fill with large delta", func(t *testing.T) {
-		c := &chunk{}
+		testChunk := &chunk{}
 
 		for i := 0; i < maxTwoBitCap; i++ {
-			assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta), i)
-			c.add(rtcp.TypeTCCPacketReceivedLargeDelta)
+			assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta), i)
+			testChunk.add(rtcp.TypeTCCPacketReceivedLargeDelta)
 		}
 
-		assert.Equal(t, c.deltas, []uint16{2, 2, 2, 2, 2, 2, 2})
-		assert.True(t, c.hasLargeDelta)
-		assert.False(t, c.hasDifferentTypes)
+		assert.Equal(t, testChunk.deltas, []uint16{2, 2, 2, 2, 2, 2, 2})
+		assert.True(t, testChunk.hasLargeDelta)
+		assert.False(t, testChunk.hasDifferentTypes)
 
-		assert.False(t, c.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
-		assert.False(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
+		assert.False(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
+		assert.False(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
 
-		statusChunk := c.encode()
+		statusChunk := testChunk.encode()
 		assert.IsType(t, &rtcp.RunLengthChunk{}, statusChunk)
 
 		buf, err := statusChunk.Marshal()
@@ -89,27 +93,27 @@ func Test_chunk_add(t *testing.T) {
 	})
 
 	t.Run("fill with different types", func(t *testing.T) {
-		c := &chunk{}
+		testChunk := &chunk{}
 
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
-		c.add(rtcp.TypeTCCPacketReceivedSmallDelta)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
-		c.add(rtcp.TypeTCCPacketReceivedSmallDelta)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
-		c.add(rtcp.TypeTCCPacketReceivedSmallDelta)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
-		c.add(rtcp.TypeTCCPacketReceivedSmallDelta)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
+		testChunk.add(rtcp.TypeTCCPacketReceivedSmallDelta)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
+		testChunk.add(rtcp.TypeTCCPacketReceivedSmallDelta)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
+		testChunk.add(rtcp.TypeTCCPacketReceivedSmallDelta)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
+		testChunk.add(rtcp.TypeTCCPacketReceivedSmallDelta)
 
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
-		c.add(rtcp.TypeTCCPacketReceivedLargeDelta)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
-		c.add(rtcp.TypeTCCPacketReceivedLargeDelta)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
-		c.add(rtcp.TypeTCCPacketReceivedLargeDelta)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
+		testChunk.add(rtcp.TypeTCCPacketReceivedLargeDelta)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
+		testChunk.add(rtcp.TypeTCCPacketReceivedLargeDelta)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
+		testChunk.add(rtcp.TypeTCCPacketReceivedLargeDelta)
 
-		assert.False(t, c.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
+		assert.False(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
 
-		statusChunk := c.encode()
+		statusChunk := testChunk.encode()
 		assert.IsType(t, &rtcp.StatusVectorChunk{}, statusChunk)
 
 		buf, err := statusChunk.Marshal()
@@ -118,38 +122,38 @@ func Test_chunk_add(t *testing.T) {
 	})
 
 	t.Run("overfill and encode", func(t *testing.T) {
-		c := chunk{}
+		testChunk := chunk{}
 
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
-		c.add(rtcp.TypeTCCPacketReceivedSmallDelta)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
-		c.add(rtcp.TypeTCCPacketNotReceived)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
-		c.add(rtcp.TypeTCCPacketNotReceived)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
-		c.add(rtcp.TypeTCCPacketNotReceived)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
-		c.add(rtcp.TypeTCCPacketNotReceived)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
-		c.add(rtcp.TypeTCCPacketNotReceived)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
-		c.add(rtcp.TypeTCCPacketNotReceived)
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketNotReceived))
-		c.add(rtcp.TypeTCCPacketNotReceived)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedSmallDelta))
+		testChunk.add(rtcp.TypeTCCPacketReceivedSmallDelta)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
+		testChunk.add(rtcp.TypeTCCPacketNotReceived)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
+		testChunk.add(rtcp.TypeTCCPacketNotReceived)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
+		testChunk.add(rtcp.TypeTCCPacketNotReceived)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
+		testChunk.add(rtcp.TypeTCCPacketNotReceived)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
+		testChunk.add(rtcp.TypeTCCPacketNotReceived)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
+		testChunk.add(rtcp.TypeTCCPacketNotReceived)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketNotReceived))
+		testChunk.add(rtcp.TypeTCCPacketNotReceived)
 
-		assert.False(t, c.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
+		assert.False(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
 
-		statusChunk1 := c.encode()
+		statusChunk1 := testChunk.encode()
 		assert.IsType(t, &rtcp.StatusVectorChunk{}, statusChunk1)
-		assert.Equal(t, 1, len(c.deltas))
+		assert.Equal(t, 1, len(testChunk.deltas))
 
-		assert.True(t, c.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
-		c.add(rtcp.TypeTCCPacketReceivedLargeDelta)
+		assert.True(t, testChunk.canAdd(rtcp.TypeTCCPacketReceivedLargeDelta))
+		testChunk.add(rtcp.TypeTCCPacketReceivedLargeDelta)
 
-		statusChunk2 := c.encode()
+		statusChunk2 := testChunk.encode()
 		assert.IsType(t, &rtcp.StatusVectorChunk{}, statusChunk2)
 
-		assert.Equal(t, 0, len(c.deltas))
+		assert.Equal(t, 0, len(testChunk.deltas))
 
 		assert.Equal(t, &rtcp.StatusVectorChunk{
 			SymbolSize: rtcp.TypeTCCSymbolSizeTwoBit,
@@ -174,23 +178,23 @@ func Test_feedback(t *testing.T) {
 	})
 
 	t.Run("add received 1", func(t *testing.T) {
-		f := &feedback{}
-		f.setBase(1, 1000*1000)
+		testFeedback := &feedback{}
+		testFeedback.setBase(1, 1000*1000)
 
-		got := f.addReceived(1, 1023*1000)
+		got := testFeedback.addReceived(1, 1023*1000)
 
 		assert.True(t, got)
-		assert.Equal(t, uint16(2), f.nextSequenceNumber)
-		assert.Equal(t, int64(15), f.refTimestamp64MS)
+		assert.Equal(t, uint16(2), testFeedback.nextSequenceNumber)
+		assert.Equal(t, int64(15), testFeedback.refTimestamp64MS)
 
-		got = f.addReceived(4, 1086*1000)
+		got = testFeedback.addReceived(4, 1086*1000)
 		assert.True(t, got)
-		assert.Equal(t, uint16(5), f.nextSequenceNumber)
-		assert.Equal(t, int64(15), f.refTimestamp64MS)
+		assert.Equal(t, uint16(5), testFeedback.nextSequenceNumber)
+		assert.Equal(t, int64(15), testFeedback.refTimestamp64MS)
 
-		assert.True(t, f.lastChunk.hasDifferentTypes)
-		assert.Equal(t, 4, len(f.lastChunk.deltas))
-		assert.NotContains(t, f.lastChunk.deltas, rtcp.TypeTCCPacketReceivedLargeDelta)
+		assert.True(t, testFeedback.lastChunk.hasDifferentTypes)
+		assert.Equal(t, 4, len(testFeedback.lastChunk.deltas))
+		assert.NotContains(t, testFeedback.lastChunk.deltas, rtcp.TypeTCCPacketReceivedLargeDelta)
 	})
 
 	t.Run("add received 2", func(t *testing.T) {
@@ -253,6 +257,53 @@ func Test_feedback(t *testing.T) {
 		}
 	})
 
+	t.Run("add received small deltas", func(t *testing.T) {
+		f := newFeedback(0, 0, 0)
+		base := int64(320 * 1000)
+		deltaUS := int64(200)
+		f.setBase(5, base)
+
+		for i := int64(0); i < 5; i++ {
+			got := f.addReceived(5+uint16(i+1), base+deltaUS*i) //nolint:gosec // G115
+			assert.True(t, got)
+		}
+
+		pkt := f.getRTCP()
+
+		expectedDeltas := []*rtcp.RecvDelta{
+			{
+				Type:  rtcp.TypeTCCPacketReceivedSmallDelta,
+				Delta: 0,
+			},
+			{
+				Type: rtcp.TypeTCCPacketReceivedSmallDelta,
+				// NOTE: The delta is less than the scale factor, but it should be rounded up.
+				// (rtcp.RecvDelta).Marshal() simply truncates to an interval of the scale factor,
+				// so we want to make sure that the deltas have any rounding applied when building
+				// the feedback.
+				Delta: 1 * rtcp.TypeTCCDeltaScaleFactor,
+			},
+			{
+				Type:  rtcp.TypeTCCPacketReceivedSmallDelta,
+				Delta: 1 * rtcp.TypeTCCDeltaScaleFactor,
+			},
+			{
+				Type: rtcp.TypeTCCPacketReceivedSmallDelta,
+				// NOTE: This is zero because even though the deltas are all the same, the rounding error has
+				// built up enough by this packet to cause it to be rounded down.
+				Delta: 0 * rtcp.TypeTCCDeltaScaleFactor,
+			},
+			{
+				Type:  rtcp.TypeTCCPacketReceivedSmallDelta,
+				Delta: 1 * rtcp.TypeTCCDeltaScaleFactor,
+			},
+		}
+		assert.Equal(t, len(expectedDeltas), len(pkt.RecvDeltas))
+		for i, d := range expectedDeltas {
+			assert.Equal(t, d, pkt.RecvDeltas[i])
+		}
+	})
+
 	t.Run("add received wrapped sequence number", func(t *testing.T) {
 		f := newFeedback(0, 0, 0)
 		f.setBase(65535, 320*1000)
@@ -352,6 +403,8 @@ func Test_feedback(t *testing.T) {
 }
 
 func addRun(t *testing.T, r *Recorder, sequenceNumbers []uint16, arrivalTimes []int64) {
+	t.Helper()
+
 	assert.Equal(t, len(sequenceNumbers), len(arrivalTimes))
 
 	for i := range sequenceNumbers {
@@ -365,21 +418,29 @@ const (
 
 func increaseTime(arrivalTime *int64, increaseAmount int64) int64 {
 	*arrivalTime += increaseAmount
+
 	return *arrivalTime
 }
 
 func marshalAll(t *testing.T, pkts []rtcp.Packet) {
+	t.Helper()
+
 	for _, pkt := range pkts {
-		_, err := pkt.Marshal()
+		marshaled, err := pkt.Marshal()
 		assert.NoError(t, err)
+
+		// Chrome expects feedback packets to always be 18 bytes or more.
+		// https://source.chromium.org/chromium/chromium/src/+/main:third_party/webrtc/modules/rtp_rtcp/source/rtcp_packet/transport_feedback.cc;l=423?q=transport_feedback.cc&ss=chromium%2Fchromium%2Fsrc
+		//nolint:lll
+		assert.GreaterOrEqual(t, len(marshaled), 18)
 	}
 }
 
 func TestBuildFeedbackPacket(t *testing.T) {
-	r := NewRecorder(5000)
+	recoder := NewRecorder(5000)
 
 	arrivalTime := int64(scaleFactorReferenceTime)
-	addRun(t, r, []uint16{0, 1, 2, 3, 4, 5, 6, 7}, []int64{
+	addRun(t, recoder, []uint16{0, 1, 2, 3, 4, 5, 6, 7}, []int64{
 		scaleFactorReferenceTime,
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
@@ -390,7 +451,7 @@ func TestBuildFeedbackPacket(t *testing.T) {
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor*256),
 	})
 
-	rtcpPackets := r.BuildFeedbackPacket()
+	rtcpPackets := recoder.BuildFeedbackPacket()
 	assert.Equal(t, 1, len(rtcpPackets))
 
 	assert.Equal(t, &rtcp.TransportLayerCC{
@@ -433,24 +494,25 @@ func TestBuildFeedbackPacket(t *testing.T) {
 }
 
 func TestBuildFeedbackPacket_Rolling(t *testing.T) {
-	r := NewRecorder(5000)
+	recoder := NewRecorder(5000)
 
 	arrivalTime := int64(scaleFactorReferenceTime)
-	addRun(t, r, []uint16{65535}, []int64{
+	addRun(t, recoder, []uint16{65534, 65535}, []int64{
 		arrivalTime,
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
 	})
 
-	rtcpPackets := r.BuildFeedbackPacket()
+	rtcpPackets := recoder.BuildFeedbackPacket()
 	assert.Equal(t, 1, len(rtcpPackets))
 
-	addRun(t, r, []uint16{4, 8, 9, 10}, []int64{
+	addRun(t, recoder, []uint16{0, 4, 5, 6}, []int64{
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
 	})
 
-	rtcpPackets = r.BuildFeedbackPacket()
+	rtcpPackets = recoder.BuildFeedbackPacket()
 	assert.Equal(t, 1, len(rtcpPackets))
 
 	assert.Equal(t, &rtcp.TransportLayerCC{
@@ -462,7 +524,7 @@ func TestBuildFeedbackPacket_Rolling(t *testing.T) {
 		},
 		SenderSSRC:         5000,
 		MediaSSRC:          5000,
-		BaseSequenceNumber: 4,
+		BaseSequenceNumber: 0,
 		ReferenceTime:      1,
 		FbPktCount:         1,
 		PacketStatusCount:  7,
@@ -482,7 +544,7 @@ func TestBuildFeedbackPacket_Rolling(t *testing.T) {
 			},
 		},
 		RecvDeltas: []*rtcp.RecvDelta{
-			{Type: rtcp.TypeTCCPacketReceivedSmallDelta, Delta: rtcp.TypeTCCDeltaScaleFactor},
+			{Type: rtcp.TypeTCCPacketReceivedSmallDelta, Delta: rtcp.TypeTCCDeltaScaleFactor * 2},
 			{Type: rtcp.TypeTCCPacketReceivedSmallDelta, Delta: rtcp.TypeTCCDeltaScaleFactor},
 			{Type: rtcp.TypeTCCPacketReceivedSmallDelta, Delta: rtcp.TypeTCCDeltaScaleFactor},
 			{Type: rtcp.TypeTCCPacketReceivedSmallDelta, Delta: rtcp.TypeTCCDeltaScaleFactor},
@@ -491,7 +553,7 @@ func TestBuildFeedbackPacket_Rolling(t *testing.T) {
 	marshalAll(t, rtcpPackets)
 }
 
-func TestBuildFeedbackPacketCount(t *testing.T) {
+func TestBuildFeedbackPacket_MinInput(t *testing.T) {
 	r := NewRecorder(5000)
 
 	arrivalTime := int64(scaleFactorReferenceTime)
@@ -500,16 +562,105 @@ func TestBuildFeedbackPacketCount(t *testing.T) {
 	})
 
 	pkts := r.BuildFeedbackPacket()
+	assert.Equal(t, 1, len(pkts))
+
+	assert.Equal(t, &rtcp.TransportLayerCC{
+		Header: rtcp.Header{
+			Count:   rtcp.FormatTCC,
+			Type:    rtcp.TypeTransportSpecificFeedback,
+			Length:  5,
+			Padding: true,
+		},
+		SenderSSRC:         5000,
+		MediaSSRC:          5000,
+		BaseSequenceNumber: 0,
+		ReferenceTime:      1,
+		FbPktCount:         0,
+		PacketStatusCount:  1,
+		PacketChunks: []rtcp.PacketStatusChunk{
+			&rtcp.RunLengthChunk{
+				PacketStatusSymbol: 1,
+				Type:               rtcp.TypeTCCRunLengthChunk,
+				RunLength:          1,
+			},
+		},
+		RecvDeltas: []*rtcp.RecvDelta{
+			{Type: rtcp.TypeTCCPacketReceivedSmallDelta, Delta: 0},
+		},
+	}, rtcpToTwcc(t, pkts)[0])
+	marshalAll(t, pkts)
+}
+
+func TestBuildFeedbackPacket_MissingPacketsBetweenFeedbacks(t *testing.T) {
+	recorder := NewRecorder(5000)
+
+	// Create a run of received packets.
+	arrivalTime := int64(scaleFactorReferenceTime)
+	addRun(t, recorder, []uint16{0, 1, 2, 3}, []int64{
+		scaleFactorReferenceTime,
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
+	})
+	rtcpPackets := recorder.BuildFeedbackPacket()
+	assert.Equal(t, 1, len(rtcpPackets))
+
+	// Now create another run of received packets, but with a gap.
+	addRun(t, recorder, []uint16{7, 8, 9}, []int64{
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor*256),
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
+	})
+	rtcpPackets = recorder.BuildFeedbackPacket()
+	require.Equal(t, 1, len(rtcpPackets))
+	twccPacket := rtcpToTwcc(t, rtcpPackets)[0]
+	assert.Equal(
+		t, uint16(4), twccPacket.BaseSequenceNumber,
+		"Base sequence should be one after the end of the previous feedback",
+	)
+	assert.Equal(
+		t, uint16(6), twccPacket.PacketStatusCount,
+		"Feedback should include status for both the lost and received packets",
+	)
+	expectedPacketChunks := []rtcp.PacketStatusChunk{
+		&rtcp.StatusVectorChunk{
+			Type:       rtcp.TypeTCCRunLengthChunk,
+			SymbolSize: rtcp.TypeTCCSymbolSizeTwoBit,
+			SymbolList: []uint16{
+				rtcp.TypeTCCPacketNotReceived,
+				rtcp.TypeTCCPacketNotReceived,
+				rtcp.TypeTCCPacketNotReceived,
+				rtcp.TypeTCCPacketReceivedSmallDelta,
+				rtcp.TypeTCCPacketReceivedSmallDelta,
+				rtcp.TypeTCCPacketReceivedSmallDelta,
+			},
+		},
+	}
+	assert.Equal(t, expectedPacketChunks, twccPacket.PacketChunks)
+	marshalAll(t, rtcpPackets)
+}
+
+func TestBuildFeedbackPacketCount(t *testing.T) {
+	recorder := NewRecorder(5000)
+
+	arrivalTime := int64(scaleFactorReferenceTime)
+	addRun(t, recorder, []uint16{0, 1}, []int64{
+		arrivalTime,
+		arrivalTime,
+	})
+
+	pkts := recorder.BuildFeedbackPacket()
 	assert.Len(t, pkts, 1)
 
 	twcc := rtcpToTwcc(t, pkts)[0]
 	assert.Equal(t, uint8(0), twcc.FbPktCount)
 
-	addRun(t, r, []uint16{0}, []int64{
+	addRun(t, recorder, []uint16{0, 1}, []int64{
+		arrivalTime,
 		arrivalTime,
 	})
 
-	pkts = r.BuildFeedbackPacket()
+	pkts = recorder.BuildFeedbackPacket()
 	assert.Len(t, pkts, 1)
 
 	twcc = rtcpToTwcc(t, pkts)[0]
@@ -562,10 +713,10 @@ func TestDuplicatePackets(t *testing.T) {
 
 func TestShortDeltas(t *testing.T) {
 	t.Run("SplitsOneBitDeltas", func(t *testing.T) {
-		r := NewRecorder(5000)
+		recorder := NewRecorder(5000)
 
 		arrivalTime := int64(scaleFactorReferenceTime)
-		addRun(t, r, []uint16{3, 4, 5, 7, 6, 8, 10, 11, 13, 14}, []int64{
+		addRun(t, recorder, []uint16{3, 4, 5, 7, 6, 8, 10, 11, 13, 14}, []int64{
 			arrivalTime,
 			arrivalTime,
 			arrivalTime,
@@ -578,7 +729,7 @@ func TestShortDeltas(t *testing.T) {
 			arrivalTime,
 		})
 
-		rtcpPackets := r.BuildFeedbackPacket()
+		rtcpPackets := recorder.BuildFeedbackPacket()
 		assert.Equal(t, 1, len(rtcpPackets))
 
 		pkt := rtcpToTwcc(t, rtcpPackets)[0]
@@ -683,10 +834,10 @@ func TestShortDeltas(t *testing.T) {
 }
 
 func TestReorderedPackets(t *testing.T) {
-	r := NewRecorder(5000)
+	recorder := NewRecorder(5000)
 
 	arrivalTime := int64(scaleFactorReferenceTime)
-	addRun(t, r, []uint16{3, 4, 5, 7, 6, 8, 10, 11, 13, 14}, []int64{
+	addRun(t, recorder, []uint16{3, 4, 5, 7, 6, 8, 10, 11, 13, 14}, []int64{
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
@@ -699,7 +850,7 @@ func TestReorderedPackets(t *testing.T) {
 		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
 	})
 
-	rtcpPackets := r.BuildFeedbackPacket()
+	rtcpPackets := recorder.BuildFeedbackPacket()
 	assert.Equal(t, 1, len(rtcpPackets))
 
 	pkt := rtcpToTwcc(t, rtcpPackets)[0]
@@ -751,141 +902,24 @@ func TestReorderedPackets(t *testing.T) {
 	marshalAll(t, rtcpPackets)
 }
 
-func TestInsertSorted(t *testing.T) {
-	cases := []struct {
-		l        []pktInfo
-		e        pktInfo
-		expected []pktInfo
-	}{
-		{
-			l: []pktInfo{},
-			e: pktInfo{},
-			expected: []pktInfo{{
-				sequenceNumber: 0,
-				arrivalTime:    0,
-			}},
-		},
-		{
-			l: []pktInfo{
-				{
-					sequenceNumber: 0,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 1,
-					arrivalTime:    0,
-				},
-			},
-			e: pktInfo{
-				sequenceNumber: 2,
-				arrivalTime:    0,
-			},
-			expected: []pktInfo{
-				{
-					sequenceNumber: 0,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 1,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 2,
-					arrivalTime:    0,
-				},
-			},
-		},
-		{
-			l: []pktInfo{
-				{
-					sequenceNumber: 0,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 2,
-					arrivalTime:    0,
-				},
-			},
-			e: pktInfo{
-				sequenceNumber: 1,
-				arrivalTime:    0,
-			},
-			expected: []pktInfo{
-				{
-					sequenceNumber: 0,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 1,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 2,
-					arrivalTime:    0,
-				},
-			},
-		},
-		{
-			l: []pktInfo{
-				{
-					sequenceNumber: 0,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 1,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 2,
-					arrivalTime:    0,
-				},
-			},
-			e: pktInfo{
-				sequenceNumber: 1,
-				arrivalTime:    0,
-			},
-			expected: []pktInfo{
-				{
-					sequenceNumber: 0,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 1,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 2,
-					arrivalTime:    0,
-				},
-			},
-		},
-		{
-			l: []pktInfo{
-				{
-					sequenceNumber: 10,
-					arrivalTime:    0,
-				},
-			},
-			e: pktInfo{
-				sequenceNumber: 9,
-				arrivalTime:    0,
-			},
-			expected: []pktInfo{
-				{
-					sequenceNumber: 9,
-					arrivalTime:    0,
-				},
-				{
-					sequenceNumber: 10,
-					arrivalTime:    0,
-				},
-			},
-		},
-	}
-	for i, c := range cases {
-		c := c
-		t.Run(fmt.Sprintf("%v", i), func(t *testing.T) {
-			assert.Equal(t, c.expected, insertSorted(c.l, c.e))
-		})
-	}
+func TestPacketsHheld(t *testing.T) {
+	recorder := NewRecorder(5000)
+	assert.Zero(t, recorder.PacketsHeld())
+
+	arrivalTime := int64(scaleFactorReferenceTime)
+	addRun(t, recorder, []uint16{0, 1, 2}, []int64{
+		arrivalTime,
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
+	})
+	assert.Equal(t, recorder.PacketsHeld(), 3)
+
+	addRun(t, recorder, []uint16{3, 4}, []int64{
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
+		increaseTime(&arrivalTime, rtcp.TypeTCCDeltaScaleFactor),
+	})
+	assert.Equal(t, recorder.PacketsHeld(), 5)
+
+	recorder.BuildFeedbackPacket()
+	assert.Zero(t, recorder.PacketsHeld())
 }
