@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package interceptor
 
 // Registry is a collector for interceptors.
@@ -10,13 +13,13 @@ func (r *Registry) Add(f Factory) {
 	r.factories = append(r.factories, f)
 }
 
-// Build constructs a single Interceptor from a InterceptorRegistry
+// Build constructs a single Interceptor from a InterceptorRegistry.
 func (r *Registry) Build(id string) (Interceptor, error) {
 	if len(r.factories) == 0 {
 		return &NoOp{}, nil
 	}
 
-	interceptors := []Interceptor{}
+	interceptors := make([]Interceptor, 0, len(r.factories))
 	for _, f := range r.factories {
 		i, err := f.NewInterceptor(id)
 		if err != nil {
